﻿namespace XFDemoTrail1
{
    using System;
    using System.Collections.ObjectModel;

    using ThinkGeo.MapSuite.Core;

    public class CustomUnitScaleBarAdornmentLayer : ScaleBarAdornmentLayer
    {
        /// <summary>
        /// The unit text.
        /// </summary>
        private string unitText;

        /// <summary>
        /// The meter to unit.
        /// </summary>
        private double meterToUnit;

        /// <summary>
        /// The unit round value.
        /// </summary>
        private double unitRoundValue;

        /// <summary>
        /// The width.
        /// </summary>
        private int width = 200;

        /// <summary> Initializes a new instance of the <see cref="CustomUnitScaleBarAdornmentLayer"/> class. </summary>
        /// <param name="position"> The position. </param>
        public CustomUnitScaleBarAdornmentLayer(AdornmentPosition position)
        {
            MeterToUnit = 1;
            Position = position;
            UnitText = "m";
            MaxWidth = 200;
        }

        #region Properties

        /// <summary>
        /// Gets or sets the unit text.
        /// </summary>
        public string UnitText
        {
            get
            {
                return unitText;
            }
            set
            {
                unitText = value;
            }
        }

        /// <summary>
        /// Gets or sets the max width. Maximum width of the scale bar in screen coordinate.
        /// </summary>
        public new int MaxWidth
        {
            get
            {
                return width;
            }
            set
            {
                width = value;
            }
        }

        /// <summary>
        /// Gets or sets the meter to unit. Ratio of meter to the unit.  
        /// </summary>
        public double MeterToUnit
        {
            get
            {
                return meterToUnit;
            }
            set
            {
                meterToUnit = value;
            }
        }

        /// <summary>
        /// Gets or sets the projection, Projection for the map (internal) and the projection the scale bar is based on (external)..
        /// </summary>
        public ManagedProj4Projection Projection { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether show scale text.
        /// </summary>
        public bool ShowScaleText { get; set; }

        /// <summary>
        /// Gets or sets the position.
        /// </summary>
        public AdornmentPosition Position { get; set; }

        #endregion

        /// <summary> The draw core. </summary>
        /// <param name="canvas"> The canvas. </param>
        /// <param name="labelsInAllLayers"> The labels in all layers. </param>
        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            // Set the position of the scale bar on canvas.
            int margin = 10;
            float Xpos = 0;
            float Ypos = 0;
            switch (Position)
            {
                case AdornmentPosition.UpperLeft:
                    Xpos = margin;
                    Ypos = margin;
                    break;
                case AdornmentPosition.UpperRight:
                    Xpos = (int)canvas.Width - margin;
                    Ypos = margin;
                    break;
                case AdornmentPosition.LowerLeft:
                    Xpos = margin;
                    Ypos = (int)canvas.Height - margin;
                    break;
                case AdornmentPosition.LowerRight:
                    Xpos = (int)canvas.Width - margin;
                    Ypos = (int)canvas.Height - margin;
                    break;
                case AdornmentPosition.LowerCenter:
                    Xpos = ((int)canvas.Width / 2) - margin;
                    Ypos = ((int)canvas.Height) - margin;
                    break;
                //case AdornmentPosition.Center:
                //    Xpos = ((int)canvas.Width / 2) - margin;
                //    Ypos = ((int)canvas.Height / 2) - margin;
                //    break;
                case AdornmentPosition.UpperCenter:
                    Xpos = ((int)canvas.Width / 2) + margin;
                    Ypos = margin + 10;
                    break;
            }

            //Gets the left and right location of the scale bar in world coordinate according to the maximum width and X and Y position.
            ScreenPointF screenLocation = GetDrawingLocation(canvas, width, Ypos);
            PointShape scaleBarMapPointShape = ExtentHelper.ToWorldCoordinate(
                canvas.CurrentWorldExtent,
                screenLocation.X,
                screenLocation.Y,
                canvas.Width,
                canvas.Height);
            PointShape scaleBarMapRightPointShape = ExtentHelper.ToWorldCoordinate(
                canvas.CurrentWorldExtent,
                screenLocation.X + this.width,
                screenLocation.Y,
                canvas.Width,
                canvas.Height);
            //Applies the reprojection so that the scale bar is not based on the displayed projection of the map (internal) but on the desired projection (external)
            //Typically the external projection will have distance characteristics more accurate than the displayed projection for a more reliable scalebar.
            //proj4.Open();
            //PointShape scaleBarMapPointShapeGeo = (PointShape)proj4.ConvertToExternalProjection(scaleBarMapPointShape);
            //PointShape scaleBarMapRightPointShapeGeo = (PointShape)proj4.ConvertToExternalProjection(scaleBarMapRightPointShape);
            //proj4.Close();
            //Gets the length of the scale bar according to the unit and the maximum width of the scale bar.
            double fullBarLength = scaleBarMapPointShape.GetDistanceTo(
                scaleBarMapRightPointShape,
                GeographyUnit.Meter,
                DistanceUnit.Meter);
            //Adjusts the length of the scale bar in order to have a round number.
            unitRoundValue = GetRoundValue(fullBarLength / meterToUnit);
            double barLength = ((unitRoundValue * meterToUnit) * this.width) / fullBarLength;
            //Draw the line of the scale bar according to the adjusted length.
            GeoPen pen = new GeoPen(GeoColor.StandardColors.Black, 2F);

            canvas.DrawLine(
                new ScreenPointF[]
                    {
                        new ScreenPointF(Xpos, Ypos - 10), new ScreenPointF(Xpos, Ypos),
                        new ScreenPointF((float)barLength + Xpos, Ypos),
                        new ScreenPointF((float)barLength + Xpos, Ypos - 10)
                    },
                pen,
                DrawingLevel.LevelOne,
                0,
                0);
            // Displays the text for the value and unit text.
            if (ShowScaleText)
            {
                var currentScale = GetRoundValue(canvas.CurrentScale);
                var s = $", 1:{Convert.ToString(currentScale)}";
                canvas.DrawText(
                    Convert.ToString(unitRoundValue) + " " + unitText + s,
                    new GeoFont("Arial", 8, DrawingFontStyles.Bold),
                    new GeoSolidBrush(GeoColor.StandardColors.Black),
                    new ScreenPointF[] { new ScreenPointF((float)(barLength / 2) + Xpos, Ypos - 10) },
                    DrawingLevel.LevelOne);
            }
            else
            {
                canvas.DrawText(
                    Convert.ToString(unitRoundValue) + " " + unitText,
                    new GeoFont("Arial", 8, DrawingFontStyles.Bold),
                    new GeoSolidBrush(GeoColor.StandardColors.Black),
                    new ScreenPointF[] { new ScreenPointF((float)(barLength / 2) + Xpos, Ypos - 10) },
                    DrawingLevel.LevelOne);
            }
        }

        /// <summary> Function to round down the length to fit within the maximum width of the scale bar. </summary>
        /// <param name="unitValue"> The unit value. </param>
        /// <returns> The <see cref="double"/>. </returns>
        private double GetRoundValue(double unitValue)
        {
            double result;
            double interval = GetRoundingValue(unitValue);
            result = FloorNumber(unitValue, interval);
            return result;
        }

        /// <summary> Gets the rounding value to be used according to the length of the unit value.  </summary>
        /// <param name="unitValue"> The unit value. </param>
        /// <returns> The <see cref="double"/>. </returns>
        private double GetRoundingValue(double unitValue)
        {
            double result = 0;
            if (unitValue > 100000)
            {
                result = 50000;
            }
            else if (unitValue > 10000)
            {
                result = 5000;
            }
            else if (unitValue > 1000)
            {
                result = 500;
            }
            else if (unitValue > 100)
            {
                result = 50;
            }
            else if (unitValue > 10)
            {
                result = 5;
            }
            else if (unitValue > 1)
            {
                result = 0.5;
            }
            else if (unitValue > 0.1)
            {
                result = 0.05;
            }
            else if (unitValue > 0.01)
            {
                result = 0.005;
            }

            return result;
        }

        /// <summary> Function used for rounding down a number according to the rounding value.  </summary>
        /// <param name="number"> The number. </param>
        /// <param name="rounding"> The rounding. </param>
        /// <returns> The <see cref="double"/>. </returns>
        private double FloorNumber(double number, double rounding)
        {
            double result = 0;
            double ieeeRemainder = Math.IEEERemainder(number, rounding);
            if (ieeeRemainder > 0)
            {
                result = number - ieeeRemainder;
            }
            else if (ieeeRemainder < 0)
            {
                result = (number + Math.Abs(ieeeRemainder)) - rounding;
            }
            else
            {
                result = number;
            }

            return result;
        }
    }
}